#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# (c) Rene Hadler
# rene@hadler.me
# www.hadler.me
#

import kopano
from MAPI.Util import *
import MAPI.Util
import MAPI.Time
import xmlrpc.client
import configparser
import base64
import magic
from datetime import date, timedelta, datetime
import time
import pytz
import ssl

config = configparser.ConfigParser()
config.read('/etc/odoo2kopano-tools/calendar.conf')

KOPANO_USER = config.get("IMPORT", "KOPANO_USER")
# Use Dev Folder when run in DEBUG Mode
if os.environ.get("ITEAS_DEBUG", 0):
    KOPANO_FOLDER = "odook-debug"
else:
    KOPANO_FOLDER = config.get("IMPORT", "KOPANO_FOLDER")

ODOO_SERVER = config.get("EXPORT", "ODOO_SERVER")
ODOO_DB = config.get("EXPORT", "ODOO_DB")
ODOO_API_USER = config.get("EXPORT", "ODOO_API_USER")
ODOO_API_PASS = config.get("EXPORT", "ODOO_API_PASS")
ODOO_API_LANG = config.get("EXPORT", "ODOO_API_LANG")

# Define MAPI Tag to store odoo ID
# See also: https://documentation.kopano.io/kopano_python_kopano/function_reference.html#properties | https://documentation.kopano.io/kopano_python_language_binding/getting_started.html
IT_CAL_ODOO_ID = PROP_TAG( PT_LONG, 33336)

extra = {"PR_TITLE": 3604511,
         "PR_TEXT": 268435487,
         "PR_LOCATION": 2148007967,
         "PR_DATE_START1": 2148335680,
         "PR_DATE_START2": 2176188480,
         "PR_DATE_END1": 2148401216,
         "PR_DATE_END2": 2176254016,
}

api_common = xmlrpc.client.ServerProxy('%s/xmlrpc/2/common' % ODOO_SERVER, context=ssl.create_default_context(ssl.Purpose.CLIENT_AUTH))
uid = api_common.authenticate(ODOO_DB, ODOO_API_USER, ODOO_API_PASS, {})

api_models = xmlrpc.client.ServerProxy('%s/xmlrpc/2/object' % ODOO_SERVER, context=ssl.create_default_context(ssl.Purpose.CLIENT_AUTH))
#
try:
    data = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'calendar.event', 'search_read',
        [],
        {
         'fields': ['name', 'description', 'location', 'start', 'user_id', 'duration', 'alarm_ids', 'categ_ids', 'partner_ids', 'stop', 'allday'],
         'context' :{'lang': ODOO_API_LANG}
        })
except xmlrpc.client.Fault:
    print("Fehler in der Odoo Datenabfrage!", sys.exc_info())
    sys.exit(1)

#
try:
    data2 = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'res.users', 'search_read',
        [],
        {
            'fields': ['login', 'partner_id'],
            'context' :{'lang': ODOO_API_LANG}
        })

    #user_ids = []
    #for user in data2:
    #    if user["partner_id"][0] not in user_ids:
    #        user_ids.append(user["partner_id"][0])

    data2_partner = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'res.partner', 'search_read',
        [],
        {
            'fields': ['email', 'name', 'parent_id'],
            'context' :{'lang': ODOO_API_LANG}
        })

    for user in data2:
        for partner in data2_partner:
            if user["partner_id"][0] == partner["id"]:
                user["partner"] = partner

except xmlrpc.client.Fault:
    print("Fehler in der Odoo Datenabfrage!", sys.exc_info())
    sys.exit(1)

try:
    data3 = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'calendar.alarm', 'search_read',
        [],
        {
            'fields': ['duration_minutes'],
            'context' :{'lang': ODOO_API_LANG}
        })
except xmlrpc.client.Fault:
    print("Fehler in der Odoo Datenabfrage!", sys.exc_info())
    sys.exit(1)

try:
    data4 = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'calendar.event.type', 'search_read',
        [],
        {
            'fields': ['id', 'display_name'],
            'context' :{'lang': ODOO_API_LANG}
        })
except xmlrpc.client.Fault:
    print("Fehler in der Odoo Datenabfrage!", sys.exc_info())
    sys.exit(1)

server = kopano.Server()
store = server.user(KOPANO_USER).store
if KOPANO_FOLDER == "":
    calendarstore = store.calendar
else:
    calendarstore = store.folder(KOPANO_FOLDER)

categories = {}
for category in data4:
    categories[category['id']] = category['display_name']

map_oid2entryid = {}
# Remove deleted events routines
delids = []
for item in calendarstore:
    iid = item.get_prop(IT_CAL_ODOO_ID)
    if iid:
        delids.append(iid.value)
        map_oid2entryid[iid.value] = item.entryid

for event in data:
    if event['name'] == "Busy":
        try:
            event['name'] = "Busy (%s)" % event["user_id"][1]
        except:
            pass

    if event['id'] in delids:
        delids.remove(event['id'])

print("Starting odoo2kopano-calendar cleanup")
for item in calendarstore:
    iid = item.get_prop(IT_CAL_ODOO_ID)
    if iid:
        if iid.value in delids:
            print("Delete calendar event", item)
            calendarstore.delete(item)

def has_val(obj):
    return obj if obj != False else ""

utc = pytz.timezone("UTC")

print("Starting odoo2kopano-calendar import")

for event in data:

    if event['id'] in map_oid2entryid:
        new_item = calendarstore.item(map_oid2entryid[event['id']])
    else:
        new_item = calendarstore.create_item()

    cur_attendees = []
    for a in new_item.attendees():
        if a.address.name not in cur_attendees:
            cur_attendees.append(a.address.name)

    if event['categ_ids']:
        ic = []
        for catid in event['categ_ids']:
            ic.append(categories[catid])

        new_item.categories = ic

    if event['partner_ids']:
        for attendee_id in event['partner_ids']:
            attendee = list(filter(lambda x: x['id'] == attendee_id, data2_partner))[0]
            addr_name = attendee["email"] if attendee["email"] else attendee["name"]
            if addr_name and addr_name not in cur_attendees:
                new_item.create_attendee("required", addr_name)

    new_item.mapiobj.SetProps([SPropValue(extra['PR_TITLE'], event['name'])])
    new_item.mapiobj.SetProps([SPropValue(extra['PR_TEXT'], event['description'] if event['description'] else "")])
    new_item.mapiobj.SetProps([SPropValue(extra['PR_LOCATION'], event['location'] if event['location'] else "")])

    if event['allday']:
        new_item.all_day = True

        adate_start = datetime.strptime(event['start'], "%Y-%m-%d %H:%M:%S")
        adate_end = datetime.strptime(event['stop'], "%Y-%m-%d %H:%M:%S") + timedelta(days=1)
        adate_start_f = datetime(adate_start.year, adate_start.month, adate_start.day, 0, 0, 0)
        adate_end_f = datetime(adate_end.year, adate_end.month, adate_end.day, 0, 0, 0)
        new_item.start = adate_start_f
        new_item.end = adate_end_f
    else:
        new_item.all_day = False

        time_start_date = datetime.strptime(event['start'], "%Y-%m-%d %H:%M:%S")
        time_start_utc = utc.localize(time_start_date, is_dst=None)
        value = MAPI.Time.unixtime(datetime.timestamp(time_start_utc))
        new_item.mapiobj.SetProps([SPropValue(extra['PR_DATE_START1'], value)])
        new_item.mapiobj.SetProps([SPropValue(extra['PR_DATE_START2'], value)])

        time_end_date = datetime.strptime(event['stop'], "%Y-%m-%d %H:%M:%S")
        time_start_utc = utc.localize(time_end_date, is_dst=None)
        value = MAPI.Time.unixtime(datetime.timestamp(time_start_utc))
        new_item.mapiobj.SetProps([SPropValue(extra['PR_DATE_END1'], value)])
        new_item.mapiobj.SetProps([SPropValue(extra['PR_DATE_END2'], value)])

    new_item.mapiobj.SetProps([SPropValue(IT_CAL_ODOO_ID, event['id'])])

    if event['alarm_ids']:
        alarmid = event['alarm_ids'][0]
        alarms = list(filter(lambda x: x['id'] == alarmid, data3))
        if alarms:
            alarm = alarms[0]
            new_item.reminder = True
            new_item.reminder_minutes = alarm['duration_minutes']
    else:
        new_item.reminder = False
        new_item.reminder_minutes = 0

    new_item.mapiobj.SaveChanges(KEEP_OPEN_READWRITE)

    if event['id'] in map_oid2entryid:
        print("Update Odoo event successful: %s" % (event['name']))
    else:
        print("Import Odoo event successful: %s" % (event['name']))
