#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# (c) Rene Hadler
# rene@hadler.me
# www.hadler.me
#

import kopano
from MAPI.Util import *
import MAPI.Util
import MAPI.Time
import xmlrpc.client
import configparser
import base64
import magic
from datetime import date
import time
import ssl

config = configparser.ConfigParser()
config.read('/etc/odoo2kopano-tools/contact.conf')

KOPANO_USER = config.get("IMPORT", "KOPANO_USER")
KOPANO_FOLDER = config.get("IMPORT", "KOPANO_FOLDER")

ODOO_SERVER = config.get("EXPORT", "ODOO_SERVER")
ODOO_DB = config.get("EXPORT", "ODOO_DB")
ODOO_API_USER = config.get("EXPORT", "ODOO_API_USER")
ODOO_API_PASS = config.get("EXPORT", "ODOO_API_PASS")
ODOO_API_LANG = config.get("EXPORT", "ODOO_API_LANG")

extra = {"PR_DISPLAY_NAME_FULL": 0x8130001f,
         "PR_EMAIL": 0x8133001f,
         "PR_FILE_AS": 0x80b5001f,
         "PR_WEBSITE": 0x80db001f,
         "PR_MAIL": 0x8134001f,
         "PR_ADDRESS": 0x80f5001f,
         "PR_CITY": 0x80f6001f,
         "PR_STATE": 0x80f7001f,
         "PR_ZIP": 0x80f8001f,
         "PR_COUNTRY": 0x80f9001f,
         "PR_IM": 0x8112001f,
         "PR_EMAIL2": 0X8144001F,
         "PR_MAIL2": 0X8143001F,
         "PR_DISPLAY_NAME_FULL2": 0X8140001F,
         "PR_EMAIL3": 0X8154001F,
         "PR_MAIL3": 0X8153001F,
         "PR_DISPLAY_NAME_FULL3": 0X8150001F,
         "PR_CATEGORIES": 0x850d101f,
         "PR_PRIVATE": 0x81a6000b
}

api_common = xmlrpc.client.ServerProxy('%s/xmlrpc/2/common' % ODOO_SERVER, context=ssl.create_default_context(ssl.Purpose.CLIENT_AUTH))
uid = api_common.authenticate(ODOO_DB, ODOO_API_USER, ODOO_API_PASS, {})

api_models = xmlrpc.client.ServerProxy('%s/xmlrpc/2/object' % ODOO_SERVER, context=ssl.create_default_context(ssl.Purpose.CLIENT_AUTH))
# Try with field birthday first
try:
    data = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'res.partner', 'search_read',
        [],
        {
         'fields': ['name', 'phone', 'mobile', 'email', 'street', 'zip', 'city', 'state_id', 'birthday', 'parent_id', 'comment', 'category_id', 'website', 'title', 'function', 'image_256', 'country_id'],
         'context' :{'lang': ODOO_API_LANG}
        })
except xmlrpc.client.Fault:
    data = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'res.partner', 'search_read',
        [],
        {
            'fields': ['name', 'phone', 'mobile', 'email', 'street', 'zip', 'city', 'state_id', 'parent_id', 'comment', 'category_id', 'website', 'title', 'function', 'image_256', 'country_id'],
            'context' :{'lang': ODOO_API_LANG}
        })

data2 = api_models.execute_kw(
    ODOO_DB, uid, ODOO_API_PASS, 'res.partner.category', 'search_read',
    [],
    {
     'fields': ['id', 'display_name'],
     'context' :{'lang': ODOO_API_LANG}
    })

categories = {}
for category in data2:
    categories[category['id']] = category['display_name']

server = kopano.Server()
store = server.user(KOPANO_USER).store
if KOPANO_FOLDER == "":
    contactsstore = store.contacts
else:
    contactsstore = store.folder(KOPANO_FOLDER)

contactsstore.empty()

def has_val(obj):
    return obj if obj != False else ""

for mcontact in data:

    state = mcontact['state_id'][1] if mcontact['state_id'] != False else ""
    country = mcontact['country_id'][1] if mcontact['country_id'] != False else ""
    title = mcontact['title'][1] if mcontact['title'] != False else ""
    name = mcontact['name'].split(" ") if mcontact['name'] != False else ("", "")
    if len(name) == 1:
        name.append("")

    if name[0] == "" and has_val(mcontact['email']) == "":
        print("Import failed, not enough data: %s" % mcontact)
        continue

    new_item = contactsstore.create_item()

    if mcontact['image_256']:
        mimetype = magic.detect_from_content(base64.b64decode(mcontact['image_256']))
        new_item.set_photo("avatar", base64.b64decode(mcontact['image_256']), mimetype.mime_type)
        new_item.mapiobj.SetProps([SPropValue(0X80C5000B, True)])

    if "birthday" in mcontact and mcontact['birthday']:
        d = mcontact['birthday'].split("-")
        datetime_date = date(int(d[0]), int(d[1]), int(d[2]))
        value = MAPI.Time.unixtime(time.mktime(datetime_date.timetuple()))
        new_item.mapiobj.SetProps([SPropValue(getattr(MAPI.Util, 'PR_BIRTHDAY'), value)])

    if mcontact['category_id']:
        ic = []
        for catid in mcontact['category_id']:
            ic.append(categories[catid])

        new_item.categories = ic

    new_item.mapiobj.SetProps([SPropValue(extra['PR_EMAIL'], has_val(mcontact['email']))])
    new_item.mapiobj.SetProps([SPropValue(extra['PR_FILE_AS'], has_val(mcontact['name']))])
    new_item.mapiobj.SetProps([SPropValue(extra['PR_ADDRESS'], has_val(mcontact['street']))])
    new_item.mapiobj.SetProps([SPropValue(extra['PR_ZIP'], has_val(mcontact['zip']))])
    new_item.mapiobj.SetProps([SPropValue(extra['PR_CITY'], has_val(mcontact['city']))])
    new_item.mapiobj.SetProps([SPropValue(extra['PR_STATE'], state)])
    new_item.mapiobj.SetProps([SPropValue(extra['PR_COUNTRY'], country)])
    new_item.mapiobj.SetProps([SPropValue(extra['PR_WEBSITE'], has_val(mcontact['website']))])

    new_item.mapiobj.SetProps([SPropValue(getattr(MAPI.Util,'PR_SURNAME'), name[len(name)-1].encode('utf-8'))])
    new_item.mapiobj.SetProps([SPropValue(getattr(MAPI.Util,'PR_GIVEN_NAME'), name[0].encode('utf-8'))])
    new_item.mapiobj.SetProps([SPropValue(getattr(MAPI.Util,'PR_SUBJECT'), has_val(mcontact['name']).encode('utf-8'))])
    new_item.mapiobj.SetProps([SPropValue(getattr(MAPI.Util,'PR_DISPLAY_NAME'), has_val(mcontact['name']).encode('utf-8'))])
    new_item.mapiobj.SetProps([SPropValue(getattr(MAPI.Util,'PR_BUSINESS_TELEPHONE_NUMBER'), has_val(mcontact['phone']).encode('utf-8'))])
    new_item.mapiobj.SetProps([SPropValue(getattr(MAPI.Util,'PR_MOBILE_TELEPHONE_NUMBER'), has_val(mcontact['mobile']).encode('utf-8'))])
    new_item.mapiobj.SetProps([SPropValue(getattr(MAPI.Util,'PR_BODY'), has_val(mcontact['comment']).encode('utf-8'))])
    new_item.mapiobj.SetProps([SPropValue(getattr(MAPI.Util,'PR_DISPLAY_NAME_PREFIX'), title.encode('utf-8'))])
    new_item.mapiobj.SetProps([SPropValue(getattr(MAPI.Util,'PR_TITLE'), has_val(mcontact['function']).encode('utf-8'))])

    new_item.mapiobj.SetProps([SPropValue(0x80D81003, [0]), SPropValue(0x80D90003, 1),])

    new_item.mapiobj.SaveChanges(KEEP_OPEN_READWRITE)

    full_name = "%s, %s (%s)" % (name[len(name)-1], name[0], new_item.email)
    new_item.create_prop(extra['PR_DISPLAY_NAME_FULL'], full_name)

    print("Import contact successful: %s" % full_name)
